/**
 * Define rview server endpoints
 * @function rviewEndpoints
 * @returns {RView} -  A rview instance
 */
'use strict'

const { spinalcase } = require('stringcase')
const rview = require('rview')
const proxy = require('apeman-middleware-proxy')
const SCOPE_PREFIX = /^@/
const SETTING_PREFIX = /^\$/

/** @lends rviewEndpoints */
function rviewEndpoints (modules) {
  let {
    base,
    scoped,
    settings
  } = Object.keys(modules).reduce((formed, key) => {
    let value = modules[ key ]
    if (SETTING_PREFIX.test(key)) {
      formed.settings[ key ] = value
    } else if (SCOPE_PREFIX.test(key)) {
      formed.scoped[ key ] = value
    } else {
      formed.base[ key ] = value
    }
    return formed
  }, { base: {}, scoped: {}, settings: {} })
  let { $serverMiddleware, $proxy } = settings
  delete settings.$proxy
  delete settings.$serverMiddleware
  $serverMiddleware = [].concat(
    ...Object.keys($proxy).map((pattern) => proxy(pattern, $proxy[ pattern ])),
    $serverMiddleware || [])
  return rview([
    (Object.assign({}, settings, { $serverMiddleware }, base)),
    ...Object.keys(scoped).map((name) => {
      let $pathname = `${settings.$pathname}/${spinalcase(name.replace(SCOPE_PREFIX, '')).trim()}`
      return Object.assign({}, settings, { $pathname }, scoped[ name ])
    })
  ])
}

module.exports = rviewEndpoints
